#ifndef INCLUDE_GKRELLM_PMU_H
#define INCLUDE_GKRELLM_PMU_H
/* ----------------------------------------------------------------------------
 * gkrellm-pmu.h
 * prototypes and definitions for gkrellm-pmu.c.
 * Copyright (c) 2003 Matthias Grimm.
 *
 * based on gkrellm-pmu version 1.4 from Joseph Palani Garcia
 * last changed on 22. August 2000 from Joseph Palani Garcia
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef ENABLE_PBB
#  include <pbb.h>
#  ifndef TAG_BATTERYPRESENT
#    define TAG_BATTERYPRESENT	(3 << 16 | 18)
#  endif
#  ifndef TAG_CHARGETIME
#    define TAG_CHARGETIME		(7 << 16 | 29)
#  endif
#  ifndef SYSINFO_HAVEIBAM
#    define SYSINFO_HAVEIBAM	(1<<7)
#  endif
#endif

#include <gkrellm2/gkrellm.h>

#if !(GKRELLM_CHECK_VERSION(2,2,0))
#  define gkrellm_gdk_string_width	gdk_string_width
#endif
		
#define PMU_PLACEMENT   MON_APM
#define PMU_STYLE_NAME  "pmu"
#define PMU_CONFIG_NAME	"PMU"
#define PMU_CONFIG_KEY	"pmu"

#ifndef STDBUFFERLEN
#  define STDBUFFERLEN	120
#endif
#define MAXBATTERIES	2
#define HORZ_MARGIN	2
#define VERT_MARGIN	1

#define BATT_PRESENT	1<<0   /* battery flags */
#define BATT_CHARGING	1<<1

enum {BATT_NA, BATT_EMPTY, BATT_LOW, BATT_GOOD, BATT_CHARGE, BATT_AC, BATT_COUNT};
#define N_BATTERY_DECALS (BATT_COUNT)

enum {DM_NONE, DM_PERCENT, DM_TIME, DM_VOLTAGE, DM_CURRENT, DM_LAST};

struct pmuconfig {
	struct {
		unsigned int use_ibam:1;
		unsigned int show_batt_na:1;
		unsigned int sum_batteries:1;
		unsigned int show_charge_time:1;
		unsigned int show_chart:1;
		unsigned int:0;
	} flags;
	GtkWidget	*check_use_ibam;;
	GtkWidget	*check_sum_batteries;
	GtkWidget	*check_show_batt_na;
	GtkWidget	*check_show_charge_time;
};

struct batt {
	int status;          /* BATT_NA, BATT_EMPTY, ... */
	int displaymode;     /* DM_TIME, DM_PERCENT, ... */
	GkrellmKrell *krell;
	GkrellmDecal *image;
	GkrellmDecal *text;

	unsigned long flags;    /* values from /proc/pmu/battery_% below */
	unsigned int charge;
	unsigned int maxcharge;
	signed int current;
	unsigned int voltage;
	unsigned int time;
};

struct pmuinfo {
	int version;           /* kernel driver version from /proc/pmu/info */
	int firmware;           /* firmware version from /proc/pmu/info */
	char name[STDBUFFERLEN]; /* machine description derived from firmware */
	int nbatteries;         /* max battery count from /proc/pmu/info */
	struct batt battery[MAXBATTERIES]; /* info for each battery from /proc/pmu/battery_% */
	struct pmuconfig config;

	GkrellmMonitor	   *monitor;    /* main Gkrellm monitor structure */
	GtkWidget	   *vbox;
	GkrellmPanel 	   *panel;        /* battery panel */
	GkrellmChart       *chart;         /* current and voltage chart */
	GkrellmChartconfig *chartconfig;
	GkrellmPiximage    *symbols;        /* images for battery panels */
	GdkPixmap          *symbols_pixmap;
	GdkBitmap          *symbols_mask;
	
	int pbbSupport;          /* =1, if PBButtonsd support is available */
	int pbbHaveIBaM;         /* =1, if PBButtonsd has IBaM support */
	int pbbBattPresent;      /* battery status from PBButtonsd */
	int pbbDischargeTime;    /* discharge time remaining from PBButtonsd */
	int pbbChargeTime;       /* charge time remaining from PBButtonsd */
	int BattPresent;         /* =0, no Battery present; =1, at least one Battery present */
	
	gint style_id;
};

/* gkrellm-pmu.c prototypes */
GkrellmMonitor *gkrellm_init_plugin (void);
void pmu_update ();
void pmu_create (GtkWidget *vbox, gint first_create);
void gkrellm_exit_plugin (void);

/* pmuproc.c prototypes */
void procpmu_init_pmuinfo (struct pmuinfo *base);
void procpmu_get_pmuinfo (struct pmuinfo *base);
int procpmu_axtoi (char *hex);
void procpmu_identify (char* buffer, int pmu);
int procpmu_update_batteryinfo (struct pmuinfo *base);

/* pmuconfig.c prototypes */
void pmuconfig_create (GtkWidget *tab_vbox);
void pmuconfig_init (struct pmuconfig *config);
void pmuconfig_sum_batteries(GtkToggleButton *button, struct pmuconfig *config);
void pmuconfig_apply ();
void pmuconfig_save (FILE *fd);
void pmuconfig_load (gchar *arg);

/* pmupanel.c prototypes */
void pmupanel_create (struct pmuinfo *base, GtkWidget *vbox, gint first_create);
void pmupanel_update (struct pmuinfo *base);
void pmupanel_updatetext (GkrellmPanel *panel, struct batt *bat, gboolean force);
gboolean pmupanel_expose (GtkWidget *widget, GdkEventExpose *ev, GkrellmPanel *panel);
gboolean pmupanel_click (GtkWidget *widget, GdkEventButton *ev, struct pmuinfo *base);

/* pmuchart.c prototypes */
void pmuchart_create (struct pmuinfo *base, GtkWidget *vbox, gint first_create);
gint pmuchart_expose (GtkWidget *widget, GdkEventExpose *ev, struct pmuinfo *base);
gint pmuchart_click (GtkWidget *widget, GdkEventButton *ev, struct pmuinfo *base);
void pmuchart_update (struct pmuinfo *base);
void pmuchart_visibility (struct pmuinfo *base);

#endif /* INCLUDE_GKRELLM_PMU_H */
