/* ----------------------------------------------------------------------------
 * GKrellM PMU plugin
 * Battery monitor for Apple Powerbooks running Linux/PPC
 * Copyright (c) 2003 Matthias Grimm
 * Version:  2.0
 *
 * based on gkrellm-pmu version 1.4 from Joseph Palani Garcia
 * last changed on 22. August 2000 from Joseph Palani Garcia
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------
 */

#include "gkrellm-pmu.h"

void
pmuchart_create (struct pmuinfo *base, GtkWidget *vbox, gint first_create)
{
	GkrellmChartdata *cd;

	if (first_create)
		base->chart = gkrellm_chart_new0();

	gkrellm_chart_create(vbox, base->monitor, base->chart, &base->chartconfig);
	cd = gkrellm_add_default_chartdata(base->chart, _("B0 current"));
	gkrellm_monotonic_chartdata(cd, FALSE);
	gkrellm_set_chartdata_draw_style_default(cd, CHARTDATA_LINE);
	cd = gkrellm_add_default_chartdata(base->chart, _("B1 current"));
	gkrellm_monotonic_chartdata(cd, FALSE);
	gkrellm_set_chartdata_draw_style_default(cd, CHARTDATA_LINE);

	gkrellm_set_chartconfig_grid_resolution(base->chartconfig, 500);
	gkrellm_set_chartconfig_auto_grid_resolution(base->chartconfig, FALSE);

	gkrellm_alloc_chartdata(base->chart);

	if (first_create) {
		gkrellm_chart_hide (base->chart, 1);
		g_signal_connect(G_OBJECT(base->chart->drawing_area),
				"expose_event", G_CALLBACK(pmuchart_expose), base);
		g_signal_connect(G_OBJECT(base->chart->drawing_area),
				"button_press_event", G_CALLBACK(pmuchart_click), base);
	} else
		pmuchart_update(base);
}

gint
pmuchart_expose (GtkWidget *widget, GdkEventExpose *ev, struct pmuinfo *base)
{
	gdk_draw_pixmap(widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		base->chart->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
		ev->area.width, ev->area.height);
	return FALSE;
}

gint
pmuchart_click (GtkWidget *widget, GdkEventButton *ev, struct pmuinfo *base)
{
	int cfg = 0;

	if (ev && ev->button == 3) {
		gkrellm_chartconfig_window_create(base->chart);
		cfg = 1;
	}

	if (!cfg) {
		if (base->BattPresent || base->config.flags.show_batt_na) {
			base->config.flags.show_chart = !base->config.flags.show_chart;
			pmuchart_visibility(base);
		}
	}
	return FALSE;
}

void
pmuchart_update (struct pmuinfo *base)
{
	gkrellm_store_chartdata (base->chart, 0,
		abs (base->battery[0].current),
		abs (base->battery[1].current));
	gkrellm_draw_chartdata (base->chart);
	gkrellm_draw_chart_to_screen(base->chart);
}

void
pmuchart_visibility (struct pmuinfo *base)
{
	if ((base->BattPresent || base->config.flags.show_batt_na)
	      && base->config.flags.show_chart)
		gkrellm_chart_show (base->chart, TRUE);
	else
		gkrellm_chart_hide (base->chart, TRUE);
	
#ifdef DEBUG
	printf("BattPresent=%d, show_batt_na=%d, show_chart=%d, Logic=%d\n",
		base->BattPresent, base->config.flags.show_batt_na, base->config.flags.show_chart,
		(base->BattPresent || base->config.flags.show_batt_na) && base->config.flags.show_chart);
#endif
}	

