/* ----------------------------------------------------------------------------
 * GKrellM PMU plugin
 * Battery monitor for Apple Powerbooks running Linux/PPC
 * Copyright (c) 2003 Matthias Grimm
 * Version:  2.0
 *
 * based on gkrellm-pmu version 1.4 from Joseph Palani Garcia
 * last changed on 22. August 2000 from Joseph Palani Garcia
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------
 */

#include "gkrellm-pmu.h"

extern struct pmuinfo pmu_data;

void
pmuconfig_create (GtkWidget *tab_vbox)
{
	struct pmuinfo *base = &pmu_data;
	char buffer[STDBUFFERLEN];
	char *authors[] = {
		N_("<h>Authors\n"),
		N_("Version 2.x written by Matthias Grimm\n"),
		N_("based on Gkrellm-PMU 1.4 written by Joseph Palani Garcia.\n\n"),
		N_("Released under the General Public Licence\n\n")
		};
	char *help[] = {
		N_("<h>Program description\n"),
		N_("GKrellm PMU creates a battery display for Apple Powerbook without using APM\n"),
		N_("frontends, which tend to be less verbose.\n\n"),
		N_("<h>Interface Description\n"),
		N_("On the left is a battery panel similar to those used in the Control Strip.\n"),
		N_("The battery symbol will change color and fill grade by current charge. The\n"),
		N_("percent value is also shown in detail using a analogue meter.\n"),
		N_("Possible symbols might occur:\n"),
		N_("<i>\tGreen and full gauche\t\t\t-\t100%\n"),
		N_("<i>\tGreen and 80% gauche\t\t\t-\t20-99%\n"),
		N_("<i>\tYellow and half gauche\t\t\t-\t1-20%\n"),
		N_("<i>\tRed and nearly empty gauche\t-\t< 1%\n"),
		N_("<i>\ttransparent\t\t\t\t\t-\tnot installed\n\n"),
		N_("Also, additional icons are shown for status:\n"),
		N_("<i>\tBolt\t-\tcharging\n"),
		N_("<i>\tPlug\t-\tplugged in (and not charging)\n"),
		N_("<i>\t+/-\t-\tin use\n\n"),
		N_("Beside this symbols more verbose status information is shown. Clicking on\n"),
		N_("them with the left mouse button will cycle through different values. The\n"),
		N_("settings will be saved on exit.\n"),
		N_("<i>\tTime Remaining (-:-- when inactive)\n"),
		N_("<i>\tTotal Percent remaining\n"),
		N_("<i>\tTotal Draw in mA. (negative is in use, positive is charging)\n"),
		N_("<i>\tTotal Voltage (in volts)\n\n"),
		N_("The values will be displayed independently for each battery unless the\n"),
		N_("option 'sum_batteries' is set.\n\n"),
		N_("If the option 'Use IBaM' is set, the time remaining will automatically\n"),
		N_("replaced by the value delivered by PBButtonsd. Currently IBaM can't handle\n"),
		N_("multiple batteries seperatly, so that this option is only available if\n"),
		N_("'sum_batteries' has been activated. With IBaM enabled GKrellm-PMU always\n"),
		N_("shows the remaining time on battery even if the batteries are charging.\n"),
		N_("If battery charging is in progress the displayed time is equivalent to\n"),
		N_("that time the computer would run on battery if the charging process was\n"),
		N_("interrupted now. If you want to see the time until the batteries are\n"),
		N_("fully recharged here, set the option 'Show Charge Time'.\n\n"),
		N_("A click on the panel with the right mouse button will open the config\n"),
		N_("window. A click with the middle mouse button will expand display to reveal\n"),
		N_("a battery load meter. Useful to visually understand relative changes.\n\n"),
		N_("A right-click on the chart opens the chart config window to adjust various\n"),
		N_("parameters of the chart. The settings will be saved. A click with any other\n"),
		N_("mouse button will hide the chart again.\n\n"),
		N_("This plugin can be themed. See the included 'Theming' file.\n\n"),
		N_("<h>Configurable Options: \n\n"),
		N_("<b>[Show Charge time]\n"),
		N_("If this Option is set, the time display shows the time remaining when\n"),
		N_("charging rather than remaining inactive. For those really impatient people\n"),
		N_("who absolutely must know.\n\n"),
		N_("<b>[Show Batt NA]\n"),
		N_("If the computer supports mores than one battery and this option is set\n"),
		N_("all batteries will be shown in the panel even one ore more are not\n"),
		N_("available at the moment.\n\n"),
		N_("<b>[Sum Batteries]\n"),
		N_("With this option set all batteries in the system will be summed up and\n"),
		N_("will be shown as one single battery.\n\n"),
		N_("<b>[Use IBaM]\n"),
		N_("With this option set, the charge and discharge time values will be queried\n"),
		N_("from PBButtonsd, if available, instead of reading /proc/pmu/battery*.\n"),
		N_("PBButtonsd uses IBaM for calculating much more accurate time estimations\n"),
		N_("than /proc/pmu/battery*.\n\n")
};

	GtkWidget	*tabs, *text;
	GtkWidget	*vbox, *vbox1;

	tabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

	/* -- Options tab */
	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Options"));
	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Panel Configuration"), 4, FALSE, 0, 2);

	gkrellm_gtk_check_button_connected(vbox1, &base->config.check_sum_batteries,
		base->config.flags.sum_batteries, FALSE, FALSE, 0,
		pmuconfig_sum_batteries, &base->config, _("sum up all batteries"));
	gkrellm_gtk_check_button(vbox1, &base->config.check_use_ibam,
		base->config.flags.use_ibam,
		FALSE, 0, _("show PBButtons' IBaM charge and discharge time estimations"));
	gkrellm_gtk_check_button(vbox1, &base->config.check_show_batt_na,
		base->config.flags.show_batt_na & !base->config.flags.sum_batteries,
		FALSE, 0, _("show also batteries currently not available"));
	gkrellm_gtk_check_button(vbox1, &base->config.check_show_charge_time,
		base->config.flags.show_charge_time,
		FALSE, 0, _("show charge time, too"));
	gtk_widget_set_sensitive (base->config.check_show_batt_na,
		!base->config.flags.sum_batteries);
	if (base->pbbHaveIBaM == 0)
		gtk_widget_set_sensitive (base->config.check_use_ibam, FALSE);
		
	/* -- Info tab */
	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
	text = gkrellm_gtk_scrolled_text_view(vbox, NULL,
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gkrellm_gtk_text_view_append(text, _("<h>System Information\n"));
	sprintf(buffer, _("GKrellm-PMU version:\t%s\n"), VERSION);
	gkrellm_gtk_text_view_append(text, buffer);
	sprintf(buffer, _("PMU driver:\t\t\t%d\n"), base->version);
	gkrellm_gtk_text_view_append(text, buffer);
	sprintf(buffer, _("PMU firmware:\t\t\t%d (%s)\n"), base->firmware, base->name);
	gkrellm_gtk_text_view_append(text, buffer);
	sprintf(buffer, _("Batteries:\t\t\t\t%d possible, %d plugged in\n\n"),
		base->nbatteries, (base->battery[0].status == BATT_NA ? 0 : 1) +
						  (base->battery[1].status == BATT_NA ? 0 : 1));
	gkrellm_gtk_text_view_append(text, buffer);
	gkrellm_gtk_text_view_append_strings(text, help,
		sizeof(help) / sizeof(char*));
	gkrellm_gtk_text_view_append_strings(text, authors,
		sizeof(authors) / sizeof(char*));
}

void
pmuconfig_init (struct pmuconfig *config)
{
	config->flags.sum_batteries    = 1;
	config->flags.use_ibam         = 0;
	config->flags.show_batt_na     = 0;
	config->flags.show_charge_time = 0;
	config->flags.show_chart       = 1;
}

void
pmuconfig_sum_batteries(GtkToggleButton *button, struct pmuconfig *config)
{
	struct pmuinfo *base = &pmu_data;
	
	gboolean status = gtk_toggle_button_get_active (button);

	if (base->pbbHaveIBaM == 1)
		gtk_widget_set_sensitive (config->check_use_ibam, status);

	gtk_widget_set_sensitive (config->check_show_batt_na, !status);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(config->check_show_batt_na),
		config->flags.show_batt_na & !status);
}

void
pmuconfig_apply ()
{
	struct pmuinfo *base = &pmu_data;
	gboolean new, rebuild_panel = 0;

	new = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(base->config.check_sum_batteries));
	if (new != base->config.flags.sum_batteries) {
		base->config.flags.sum_batteries = new;
		rebuild_panel = 1;
	}

	new = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(base->config.check_use_ibam));
	if (new != base->config.flags.use_ibam) {
		base->config.flags.use_ibam = new;
		rebuild_panel = 1;
	}

	new = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(base->config.check_show_batt_na));
	if (new != base->config.flags.show_batt_na) {
		base->config.flags.show_batt_na = new;
		rebuild_panel = 1;
	}

	base->config.flags.show_charge_time = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(base->config.check_show_charge_time));

	if (rebuild_panel) {
		procpmu_update_batteryinfo (base);    /* read /proc/pmu/battery_% */
		gkrellm_panel_destroy(base->panel);    /* battery config has changed */
		pmupanel_create (base, base->vbox, 1);  /* so rebuild the panel */
		pmupanel_update (base);
		pmuchart_visibility (base);
	}
}

void
pmuconfig_save (FILE *fd)
{
	struct pmuinfo *base = &pmu_data;

	fprintf (fd, "%s dmode0 %d\n",
		PMU_CONFIG_KEY, base->battery[0].displaymode);
	fprintf (fd, "%s dmode1 %d\n",
		PMU_CONFIG_KEY, base->battery[1].displaymode);

	fprintf (fd, "%s show_batt_na %d\n",
		PMU_CONFIG_KEY, base->config.flags.show_batt_na);
	fprintf (fd, "%s sum_batteries %d\n",
		PMU_CONFIG_KEY, base->config.flags.sum_batteries);
	fprintf (fd, "%s use_ibam %d\n",
		PMU_CONFIG_KEY, base->config.flags.use_ibam);
	fprintf (fd, "%s show_charge_time %d\n",
		PMU_CONFIG_KEY, base->config.flags.show_charge_time);
	fprintf (fd, "%s show_chart %d\n",
		PMU_CONFIG_KEY, base->config.flags.show_chart);

	/* Save any chart config changes the user has made. */
	gkrellm_save_chartconfig(fd, base->chartconfig, PMU_CONFIG_KEY, NULL);
}

void
pmuconfig_load (gchar *arg)
{
	struct pmuinfo *base = &pmu_data;
	gchar key[64], data[512];
	int val;

	if (sscanf (arg, "%s %[^\n]", key, data) == 2) {
		if (strcmp (key, "dmode0") == 0)
			sscanf (data, "%d", &base->battery[0].displaymode);
		else if (strcmp (key, "dmode1") == 0)
			sscanf (data, "%d", &base->battery[1].displaymode);
		else if (strcmp (key, "show_batt_na") == 0) {
			sscanf (data, "%d", &val);
			base->config.flags.show_batt_na = val;
		} else if (strcmp (key, "sum_batteries") == 0) {
			sscanf (data, "%d", &val);
			base->config.flags.sum_batteries = val;
		} else if (strcmp (key, "use_ibam") == 0) {
			sscanf (data, "%d", &val);
			base->config.flags.use_ibam = val;
		} else if (strcmp (key, "show_charge_time") == 0) {
			sscanf (data, "%d", &val);
			base->config.flags.show_charge_time = val;
		} else if (strcmp (key, "show_chart") == 0) {
			sscanf (data, "%d", &val);
			base->config.flags.show_chart = val;
		} else if (!strcmp(key, GKRELLM_CHARTCONFIG_KEYWORD)) {
			gkrellm_load_chartconfig(&base->chartconfig, data, 2);
		}
	}
}
