/* ----------------------------------------------------------------------------
 * GKrellM PMU plugin
 * Battery monitor for Apple Powerbooks running Linux/PPC
 * Copyright (c) 2003 Matthias Grimm
 * Version:  2.0
 *
 * based on gkrellm-pmu version 1.4 from Joseph Palani Garcia
 * last changed on 22. August 2000 from Joseph Palani Garcia
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------
 */

#include "gkrellm-pmu.h"

#if !GKRELLM_CHECK_VERSION(2,2,0)
#define gkrellm_gdk_string_width	gtk_string_width
#endif

void
pmupanel_create (struct pmuinfo *base, GtkWidget *vbox, gint first_create)
{
	struct batt   *bat;
	GkrellmStyle  *style;
	GkrellmMargin *margin;
	gint  n, k, x, yoff, w, h;

	if (first_create)
		base->panel = gkrellm_panel_new0();

	style = gkrellm_meter_style(base->style_id);
	margin = gkrellm_get_style_margins(style);

	for (n=0, k=0; n < base->nbatteries; n++) {
		bat = &base->battery[n];
		if (bat->status != BATT_NA || base->config.flags.show_batt_na) {

			/* add all necessary stuff to panel */
			bat->image = gkrellm_create_decal_pixmap(base->panel,
				base->symbols_pixmap, base->symbols_mask,
				N_BATTERY_DECALS, style, margin->left, -1);
			x = bat->image->x + bat->image->w + HORZ_MARGIN;
			w = gkrellm_chart_width() - x - margin->right;
			bat->text = gkrellm_create_decal_text(base->panel, "8/%",
				gkrellm_meter_textstyle(base->style_id), style, x, -1, w);
			bat->krell = gkrellm_create_krell(base->panel,
				gkrellm_krell_meter_piximage(base->style_id), style);
			gkrellm_monotonic_krell_values(bat->krell, FALSE);
			gkrellm_set_krell_full_scale(bat->krell, 100, 1);
			gkrellm_set_krell_margins(base->panel, bat->krell,
				bat->image->w + margin->left + VERT_MARGIN, 0);

			/* Position all decals with respect to each other. */
			h = bat->krell->h / bat->krell->depth + bat->krell->y0 + bat->text->h;
			bat->text->y += h - bat->text->h;
			if (bat->image->h > h)
				yoff = (bat->image->h - h) / 2 + k++ * (bat->image->h + VERT_MARGIN);
			else
				yoff = (h - bat->image->h) / 2 + k++ * (h + VERT_MARGIN);
			bat->text->y  += yoff;
			gkrellm_move_krell_yoff(base->panel, bat->krell, bat->krell->y0 + yoff);
			bat->image->y += yoff;
		}
	}

	gkrellm_panel_configure(base->panel, NULL, style);
	gkrellm_panel_create(vbox, base->monitor, base->panel);

	/* connect necessary signals */
	if (first_create) {
		g_signal_connect(G_OBJECT(base->panel->drawing_area), "expose_event",
				G_CALLBACK(pmupanel_expose), base->panel);
		g_signal_connect(G_OBJECT(base->panel->drawing_area), "button_press_event",
				G_CALLBACK(pmupanel_click), base);
	}
}

void
pmupanel_update (struct pmuinfo *base)
{
	struct batt *bat;
	struct pmuconfig *cfg = &base->config;
	int n, percent, show = 0;

	if (base->panel) {
		for (n=0; n < base->nbatteries; n++) {
			bat = &base->battery[n];
			if (bat->status != BATT_NA || cfg->flags.show_batt_na) {
				gkrellm_draw_decal_pixmap(base->panel, bat->image, bat->status);
				pmupanel_updatetext (base->panel, bat, FALSE);
				percent = bat->maxcharge ? bat->charge * 100 / bat->maxcharge : 0;
				gkrellm_update_krell(base->panel, bat->krell, percent);
				show = 1; /* panel should be shown */
			}
		}
		if (show)
			gkrellm_panel_show(base->panel);
		else
			gkrellm_panel_hide(base->panel);
		gkrellm_draw_panel_layers(base->panel);
	}
}

void
pmupanel_updatetext (GkrellmPanel *panel, struct batt *bat, gboolean force)
{
	static int last_dmode = DM_NONE;
	int	cur_dmode  = bat->displaymode;
	int	val, x, w;
	char	buffer[16];
	
	if (last_dmode != cur_dmode)
		force = TRUE;
	last_dmode = cur_dmode;

	switch (cur_dmode) {
	case DM_TIME:
		val = bat->time;
		if (val == -1)
			sprintf (buffer, _("-:--"));
		else
			snprintf(buffer, sizeof(buffer), _("%d:%02d h"), val / 60, val % 60);
		break;
	case DM_VOLTAGE:
		val = bat->voltage;
		snprintf (buffer, sizeof(buffer), _("%4.1f V"), val / 1000.0);
		break;
	case DM_CURRENT:
		val = bat->current;
		snprintf (buffer, sizeof(buffer), _("%d mA"), val);
		break;
	case DM_PERCENT:
	default:
		val = bat->maxcharge ? bat->charge * 100 / bat->maxcharge : 0;
		if (val == 99) val = 100;  /* if the battery is full, say it */
		snprintf(buffer, sizeof(buffer), _("%d%%"), val);
		break;
	}

	if (bat->text) {
		w = gkrellm_gdk_string_width (bat->text->text_style.font, buffer);
		x = (bat->text->w - w) / 2;
		if (x < 0) x = 0;
		bat->text->x_off = x;
		gkrellm_draw_decal_text(panel, bat->text, buffer, force ? -1 : val);
	}
}

/* callback functions */

gboolean
pmupanel_expose(GtkWidget *widget, GdkEventExpose *ev, GkrellmPanel *panel)
{
	gdk_draw_drawable(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE(widget)], panel->pixmap,
			ev->area.x, ev->area.y, ev->area.x, ev->area.y,
			ev->area.width, ev->area.height);
	return FALSE;
}

gboolean
pmupanel_click (GtkWidget *widget, GdkEventButton *ev, struct pmuinfo *base)
{
	int battery = 0, n, battcount = 0;

	if (ev->button == 3)
		gkrellm_open_config_window(base->monitor);
	else if (ev->button == 2)
		pmuchart_click (NULL, NULL, base);
	else {
		for (n=0; n < base->nbatteries; n++)
			if (base->battery[n].status != BATT_NA || base->config.flags.show_batt_na)
				if (!(battcount++))
					battery = n;
		if (battcount) {
			if (battcount > 1)
				battery = (int) (ev->y / (base->panel->h / battcount));
			base->battery[battery].displaymode++;
			if (base->battery[battery].displaymode == DM_LAST)
				base->battery[battery].displaymode = DM_NONE + 1;
			pmupanel_updatetext(base->panel, &base->battery[battery], TRUE);
			gkrellm_draw_panel_layers(base->panel);
			gkrellm_config_modified();
		}
	}
	return FALSE;
}
