/* ----------------------------------------------------------------------------
 * GKrellM PMU plugin
 * Battery monitor for Apple Powerbooks running Linux/PPC
 * Copyright (c) 2003 Matthias Grimm
 * Version:  2.0
 *
 * based on gkrellm-pmu version 1.4 from Joseph Palani Garcia
 * last changed on 22. August 2000 from Joseph Palani Garcia
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>
#include "gkrellm-pmu.h"

void
procpmu_init_pmuinfo (struct pmuinfo *base)
{
	int n;
	base->version	 = 0;
	base->firmware	 = 0;
	base->name[0]	 = '\0';
	base->nbatteries = 0;
	base->BattPresent= 0;
	
	for (n=0; n < MAXBATTERIES; n++) {
		base->battery[n].status    = BATT_NA;
		base->battery[n].charge    = 0;
		base->battery[n].maxcharge = 1;
		base->battery[n].current   = 0;
		base->battery[n].voltage   = 0;
		base->battery[n].time      = 0;
	}
}

void
procpmu_get_pmuinfo (struct pmuinfo *base)
{
	FILE *fd;
	char buffer[STDBUFFERLEN], *token;

	if ((fd = fopen ("/proc/pmu/info","r"))) {
		while (fgets (buffer, sizeof (buffer), fd))
			if ((token = strtok (buffer,":\n"))) {
				if (!strncmp ("PMU driver version", token, 18))
					base->version = atoi (strtok(0,":\n"));
				else if (!strncmp ("PMU firmware version", token, 20)) {
					base->firmware = axtoi (strtok(0,":\n"));
					procpmu_identify (base->name, base->firmware);
				} else if (!strncmp ("Battery count", token, 13))
					base->nbatteries = atoi (strtok(0,":\n"));
				else
					strtok (0,":\n");
			}
		fclose(fd);
	}
}

#ifndef ENABLE_PBB
/* This function converts a string to an integer. Additionally to
 * atoi() it converts also hexadecimal values
 */
int
axtoi (char *arg)
{
	int n, val, pwr=1, m, rc = 0;
	char hex[9], c;

	for (n=0,m=0; n < strlen(arg); n++)
		if (arg[n] != ' ') {
			hex[m++] = c = toupper(arg[n]);
			if (m == sizeof(hex) || c < '0' || c > 'F')
				return 0;   /* overflow or invalid */
		}
	hex[m] = '\0';  /* terminate string */

	for (n=0; n < m; n++) {
		c = hex[m-n-1];
		if ((c >= 'A') && (c <= 'F')) {
			val = c -'A' + 10;
		} else {
			val = c - '0';
		}
		rc = rc + val * pwr;
		pwr *= 16;
	}
	return rc;
}
#endif

void
procpmu_identify (char* buffer, int pmu)
{
  int idx;
  char *version[] = {
    N_("Unknown PowerBook"),
    "PowerBook 2400/3400/3500",        /*   9  */
    "G3 PowerBook Wallstreet",          /*  10  */
    "1999 G3 PowerBook Lombard",         /*  11  */
    "iBook/G3 PB Pismo/G4 PB Titanium",   /*  12  */
    N_("Wow, later than an Titanium! ;-)")
  };
  idx = pmu < 8 ? 8 : pmu - 8;
  if (idx > 5) idx = 5;
  snprintf(buffer, STDBUFFERLEN, "%s", version[idx]);
}

int
procpmu_update_batteryinfo (struct pmuinfo *base)
{
	FILE *fd;
	char buffer[STDBUFFERLEN], *token;
	int flags, charge, chargemax, current, voltage, time;
	int n, ac = 0, status, percent, rc = 0;

	if ((fd = fopen ("/proc/pmu/info","r"))) {
		while (fgets (buffer, sizeof (buffer), fd))
			if ((token = strtok (buffer,":\n"))) {
				if (!strncmp ("AC Power", token, 8))
					ac = atoi (strtok(0,":\n"));
				else
					strtok (0,":\n");
			}
		fclose(fd);
	}

	for (n=0; n < base->nbatteries; n++) {
		flags = charge = chargemax = current = voltage = time = 0;
		sprintf (buffer, "/proc/pmu/battery_%i", n);
		if ((fd = fopen (buffer, "r"))) {
			while (fgets (buffer,sizeof (buffer), fd)) {
				if ((token = strtok (buffer, ":\n"))) {
					if (!strncmp ("flags", token, 5)) {
						flags = axtoi (strtok (0, ":\n"));
					} else if (!strncmp ("charge", token, 6)) {
						charge = atoi(strtok(0, ":\n"));
					} else if (!strncmp ("max_charge", token, 9)) {
						chargemax = atoi (strtok(0,":\n"));
					} else if (!strncmp ("current", token, 7)) {
						current = atoi (strtok(0, ":\n"));
					} else if (!strncmp ("voltage", token, 7)) {
						voltage = atoi (strtok(0,":\n"));
					} else if (!strncmp ("time rem.", token, 8)) {
						time = atoi (strtok(0, ":\n"));
					} else
						strtok (0,":\n");
				}
			}
			fclose(fd);

			/* store values for later use in the chart */
			base->battery[n].flags     = flags;
			base->battery[n].charge    = charge;
			base->battery[n].maxcharge = chargemax = chargemax ? chargemax : 1;
			base->battery[n].current   = current;
			base->battery[n].voltage   = voltage;
			base->battery[n].time      = time;
		}
	}
	
	/* This loop handles special cases */
	for (n=0; n < base->nbatteries; n++) {
		/* check if /proc/pmu is reliable and if not estimate
		 * good values as replacement. 
		 * Unused battery source or drain no current so that the
		 * current of the active battery must be used to estimate
		 * the remaining time of the inactive battery.
		 */
		time = base->battery[n].time;
		current = base->battery[0].current + base->battery[1].current;

		if (time == 0 && base->battery[n].charge != 0) {
			if (base->firmware == 9) /* OHARE PMU */
				time = base->battery[n].charge * 274 / (-current);
			else
				time = base->battery[n].charge * 3600 / (-current);
		}

		if (current <= 0 || base->config.flags.show_charge_time)
			time /= 60;
		else
			time = -1;

		base->battery[n].time = time;
	}
	
	if (base->config.flags.sum_batteries) {
		for (n=1; n < base->nbatteries; n++) {
			base->battery[0].flags     |= base->battery[n].flags;
			base->battery[0].charge    += base->battery[n].charge;
			base->battery[0].maxcharge += base->battery[n].maxcharge;
			base->battery[0].current   += base->battery[n].current;

			if (base->battery[0].voltage == 0)
				base->battery[0].voltage = base->battery[n].voltage;
			else if (base->battery[n].voltage) {
				base->battery[0].voltage += base->battery[n].voltage;
				base->battery[0].voltage /= 2;
			}
			if (base->battery[n].time != -1)
				base->battery[0].time += base->battery[n].time;
			base->battery[n].flags = 0;
		}
		
		/* Only pay attention to config option 'use_ibam' if running
		 * pbbuttonsd supports IBaM. Otherwise ignore it. With IBaM
		 * the behaviour changed as follows:
		 * 
		 * mode     'show_charge_time'  display
		 * draining          X          time left on battery
		 * charging          0          time left on battery if charging
		 *                              would be interrupted now
		 *                   1          time till battery is fully charged   
		 *                  
		 * If PBButtons didn't support ChargeTime, GKrellm-PMU falls
		 * back to /proc/pmu/battery*. in this case. */
		if (base->pbbHaveIBaM && base->config.flags.use_ibam) {
			if ((base->battery[0].current > 0) && base->config.flags.show_charge_time) {
				if (base->pbbChargeTime != -1)
					base->battery[0].time = base->pbbChargeTime;	
			} else
				base->battery[0].time = base->pbbDischargeTime;	
		}
		rc = 0;  /* in this mode changes will be handled internally */
	}

	base->BattPresent = 0;
	for (n=0; n < base->nbatteries; n++) {
		status = BATT_NA;
		/* derive the battery status responsible for the icon to display */
		if (base->pbbBattPresent == 0) {
			base->battery[n].charge  = 0;
			base->battery[n].voltage = 0;
			base->battery[n].current = 0;
			base->battery[n].time    = -1;
		} else if (base->battery[n].flags & BATT_PRESENT) {
			if (ac) {
				if (base->battery[n].flags & BATT_CHARGING) {
					if (base->firmware == 12 &&
						 (base->battery[n].charge * 100 /
						  base->battery[n].maxcharge) >= 99)
						/* this is a bug workaround in PMU version 12 where
						 * the charging flag won't be cleared even the battery
						 * is full. So let us do it.
						 */
						status = BATT_AC;
					else
						status = BATT_CHARGE;
				} else
					status = BATT_AC;
			} else {
				percent = base->battery[n].charge * 100 /
					base->battery[n].maxcharge;
				if (percent > 20)
					status = BATT_GOOD;
				else if (percent > 1)
					status = BATT_LOW;
				else
					status = BATT_EMPTY;
			}
		}
		
		/* Check for Batteries present */
		if (status != BATT_NA)
			base->BattPresent = 1;
			
		/* Check, if battery config has changed */
		if ((status == BATT_NA && base->battery[n].status != BATT_NA) ||
		    (status != BATT_NA && base->battery[n].status == BATT_NA))
			rc = 1;
		
		base->battery[n].status = status;
	}
	return rc;
}

